﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;
using System.ServiceProcess;
using System.Diagnostics;
using System.Timers;


/*
 * ２台の KeiganMotor をUSB通信で同期制御します。
 */

namespace KeiganMotorKM_1_Multi_SerialTest
{
    public partial class Form1 : Form
    {
        
        public KeiganMotor_USB targetMotor1;
        public KeiganMotor_USB targetMotor2;

        public Form1()
        {
            InitializeComponent();
            Debug.Listeners.Add(new TextWriterTraceListener(Console.Out));
        }


        private void ConnectionButton_Click(object sender, EventArgs e)
        {
            //! オープンするシリアルポートをコンボボックスから取り出す.
            string portName = ComboBox1.SelectedItem.ToString();

            if (targetMotor1 == null)
            {
                
                try
                {
                    //　シリアルポートをオープンする.
                    targetMotor1 = new KeiganMotor_USB(portName);
                    targetMotor1.Open();
                    listBox1.Items.Add(targetMotor1);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            } else if (targetMotor2 == null && targetMotor1.serialPort.PortName != portName)
            {

                try
                {
                    //　シリアルポートをオープンする.
                    targetMotor2 = new KeiganMotor_USB(portName);
                    targetMotor2.Open();
                    listBox1.Items.Add(targetMotor2);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }

       
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Update_PortNames();
        }

        private void reloadButton_Click(object sender, EventArgs e)
        {
            Update_PortNames();
        }

        private void Update_PortNames()
        {
            string[] PortList = SerialPort.GetPortNames();

            ComboBox1.Items.Clear();

            foreach (string PortName in PortList)
            {
                ComboBox1.Items.Add(PortName);
            }
            if (ComboBox1.Items.Count > 0)
            {
                ComboBox1.SelectedIndex = ComboBox1.Items.Count - 1;
            }
        }

        private void DisconnectButton_Click(object sender, EventArgs e)
        {
            if(targetMotor1 != null && targetMotor1.serialPort.IsOpen == true)
            {
                // シリアルポートをクローズする
                targetMotor1.serialPort.Close();
                targetMotor1 = null;
                listBox1.Items.Remove(targetMotor1);
            }
            if (targetMotor2 != null && targetMotor2.serialPort.IsOpen == true)
            {
                // シリアルポートをクローズする
                targetMotor2.serialPort.Close();
                targetMotor2 = null;
                listBox1.Items.Remove(targetMotor2);
            }

            

        }

        private void SyncTestButton_Click(object sender, EventArgs e)
        {
            if(targetMotor1 != null && targetMotor2 != null)
            {
                //targetMotor1.Enable();
                //targetMotor2.Enable();

                targetMotor1.SpeedRpm(10);
                targetMotor2.SpeedRpm(10);

                // 3秒間隔で、36度ずつ10回回転させます

                // 方法(1) マスター（PC等）側で時間管理する方法です。タイマーを使います。    
                int num = 0;
                System.Timers.Timer timer = new System.Timers.Timer(3000); //3000ms

                // タイマーの処理
                timer.Elapsed += (sr, er) =>
                {
                    if (num < 10)
                    {
                        Console.WriteLine("5回繰り返します");
                        targetMotor1.MoveByDistanceDegree(36);
                        targetMotor2.MoveByDistanceDegree(36);
                        num++;
                    }
                    else
                    {
                        timer.Stop();
                        Console.WriteLine("処理を終了しました");
                    }
                };

                // タイマーを開始する
                timer.Start();
                

                // 方法(2) モーター側で時間管理する方法です。キューの機能を使用しています。
                // https://document.keigan-motor.com/software_dev/lowapis/queue
                // wait コマンドでは、指定時間次のコマンド実行を遅延させます
                // 問題としては、無数に繰り返すことにより、モーターの水晶クロックの誤差が蓄積されていきます


                // 実行には、上記のタイマー部分をコメントアウトし、以下のコメントアウトを解除して下さい。
                // （１）と異なり、いきなり回転から開始します
                //for (int i = 0 ; i<10 ; i++)
                //{
                //    targetMotor1.MoveByDistanceDegree(36);
                //    targetMotor2.MoveByDistanceDegree(36);
                //    targetMotor1.wait(3000); // 3000ms
                //    targetMotor2.wait(3000); // 3000ms
                //}
               

            }
            else
            {
                MessageBox.Show("Connect 2 motors");
            }
            
        }

        private void EnableButton_Click(object sender, EventArgs e)
        {
            if (targetMotor1 != null && targetMotor2 != null)
            {
                targetMotor1.Enable();
                targetMotor2.Enable();
            }
            else
            {
                MessageBox.Show("Connect 2 motors");
            }
        }
    }
}
