﻿using System;
using System.Linq;
using System.IO.Ports;
using System.Windows.Forms;


/* KeiganMotor Command API
 * 
 * Send Command to KeiganMotor
 * 
 */

public partial class KeiganMotor_USB
{
    public void readDeviceName()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0x46; // readDeviceName コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;


        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }

    }



    public void Enable()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0x51; //enable コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;


        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }


    public void Disable()
    {
        byte[] param = new byte[5];
        int size = 0;

        param[size++] = 0x50; //disable 

        GetIDBytes().CopyTo(param, size);
        size += 2;


        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void Speed(float value)
    {
        byte[] param = new byte[9];
        int size = 0;

        // Windows PC　はリトルエンディアンであることに注意。ビッグエンディアンにしてバイト列にする。
        byte[] spdArray = FloatToBytes(value);

        //パラメータのセット
        param[size++] = 0x58; //speed コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;

        spdArray.CopyTo(param, size);
        size += 4;

        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void SpeedRpm(float rpm)
    {
        Speed(rpm * (float)(2 * Math.PI / 60));
    }

    public void RunForward()
    {
        byte[] param = new byte[5];
        int size = 0;

        param[size++] = 0x60; //runForward コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void RunReverse()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0x61; //runReverse コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        //サーボの書き込み
        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }


    public void MoveToPosition(float value)
    {
        byte[] param = new byte[9];
        int size = 0;

        byte[] spdArray = FloatToBytes(value);

        param[size++] = 0x66; //moveToPosition コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;

        spdArray.CopyTo(param, size);
        size += 4;

        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void MoveToPositionDegree(float degree)
    {
        MoveToPosition(degree * (float)(Math.PI / 180));
    }

    public void MoveByDistance(float value)
    {
        byte[] param = new byte[9];
        int size = 0;

        byte[] spdArray = FloatToBytes(value);

        param[size++] = 0x68; //moveByDistance 
        GetIDBytes().CopyTo(param, size);
        size += 2;

        spdArray.CopyTo(param, size);
        size += 4;

        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void MoveByDistanceDegree(float degree)
    {
        MoveByDistance(degree * (float)(Math.PI / 180));
    }


    public void Free()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0x6C; //free コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void Stop()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0x6D; //stop コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void wait(UInt32 time_ms)
    {
        byte[] param = new byte[9];
        int size = 0;

        // Windows PC　はリトルエンディアンであることに注意。ビッグエンディアンにしてバイト列にする。
        byte[] timeArray = UInt32ToBytes(time_ms);

        //パラメータのセット
        param[size++] = 0x92; // コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;

        timeArray.CopyTo(param, size);
        size += 4;

        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    // LED状態の列挙子
    public enum LedState
    {
        Off,
        OnSolid,
        OnFlash,
        OnDim
    }

    public void Led(LedState state, byte red, byte green, byte blue)
    {
        byte[] param = new byte[9];
        int size = 0;

        //パラメータのセット
        param[size++] = 0xE0; //LED コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = (byte)state; // LedState
        param[size++] = red; //RGB
        param[size++] = green; //RGB
        param[size++] = blue; //RGB
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    // Teaching Playback
    public void EraseMotion(UInt16 index)
    {
        byte[] param = new byte[7];
        int size = 0;

        // Windows PC　はリトルエンディアンであることに注意。ビッグエンディアンにしてバイト列にする。
        byte[] indexArray = UInt16ToBytes(index);

        //パラメータのセット
        param[size++] = 0xAD; // コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;

        indexArray.CopyTo(param, size);
        size += 2;

        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void EraseAllMotion()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0xAE; // コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }


    public void PrepareTeachingMotion(UInt16 index, UInt32 time)
    {
        byte[] param = new byte[11];
        int size = 0;

        // Windows PC　はリトルエンディアンであることに注意。ビッグエンディアンにしてバイト列にする。
        byte[] indexArray = UInt16ToBytes(index);
        byte[] timeArray = UInt32ToBytes(time);

        //パラメータのセット
        param[size++] = 0xAA; // コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;

        indexArray.CopyTo(param, size);
        size += 2;

        timeArray.CopyTo(param, size);
        size += 4;

        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void StartTeachingMotionFromPrep()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0xAB; // コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void StopTeachingMotion()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0xAC; // コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }


    // LED状態の列挙子
    public enum playbackOption
    {
        startFromABS = 0,
        startFromCurrent = 1
    }

    public void StartPlaybackMotion(UInt16 index, UInt32 repeating, playbackOption option)
    {
        byte[] param = new byte[12];
        int size = 0;

        // Windows PC　はリトルエンディアンであることに注意。ビッグエンディアンにしてバイト列にする。
        byte[] indexArray = UInt16ToBytes(index);
        byte[] repeatArray = UInt32ToBytes(repeating);

        //パラメータのセット
        param[size++] = 0x85; // コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;

        indexArray.CopyTo(param, size);
        size += 2;

        repeatArray.CopyTo(param, size);
        size += 4;

        param[size++] = (byte)option;

        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void PreparePlaybackMotion(UInt16 index, UInt32 repeating, playbackOption option)
    {
        byte[] param = new byte[12];
        int size = 0;

        // Windows PC　はリトルエンディアンであることに注意。ビッグエンディアンにしてバイト列にする。
        byte[] indexArray = UInt16ToBytes(index);
        byte[] repeatArray = UInt32ToBytes(repeating);

        //パラメータのセット
        param[size++] = 0x86; // コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;

        indexArray.CopyTo(param, size);
        size += 2;

        repeatArray.CopyTo(param, size);
        size += 4;

        param[size++] = (byte)option;

        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void StartPlaybackMotionFromPrep()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0x87; // コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void StopPlaybackMotion()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0x88; // コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void EraseTaskset(UInt16 index)
    {
        byte[] param = new byte[7];
        int size = 0;

        // Windows PC　はリトルエンディアンであることに注意。ビッグエンディアンにしてバイト列にする。
        byte[] indexArray = UInt16ToBytes(index);

        //パラメータのセット
        param[size++] = 0xA3; // コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;

        indexArray.CopyTo(param, size);
        size += 2;

        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void EraseAllTasksets()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0xA4; // コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    /* 
     * Registers
     */
  
    public void TeachingInterval(UInt32 interval)
    {
        byte[] param = new byte[9];
        int size = 0;

        // Windows PC　はリトルエンディアンであることに注意。ビッグエンディアンにしてバイト列にする。
        byte[] intvlArray = UInt32ToBytes(interval);

        //パラメータのセット
        param[size++] = 0x16; // コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;

        intvlArray.CopyTo(param, size);
        size += 4;

        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void PlaybackInterval(UInt32 interval)
    {
        byte[] param = new byte[9];
        int size = 0;

        // Windows PC　はリトルエンディアンであることに注意。ビッグエンディアンにしてバイト列にする。
        byte[] intvlArray = UInt32ToBytes(interval);

        //パラメータのセット
        param[size++] = 0x17; // コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;

        intvlArray.CopyTo(param, size);
        size += 4;

        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }


    /*
     * Others
     */
    public void Reboot()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0xF0; // コマンド
        GetIDBytes().CopyTo(param, size);
        size += 2;
        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true) 
        {
            serialPort.Write(param, 0, size);
        }
    }

    public void EnableCheckSum(bool isEnabled)
    {
        byte[] param = new byte[6];
        int size = 0;

        //パラメータのセット
        param[size++] = 0xF3; // コマンド

        //GetIDBytes().CopyTo(param, size);
        param[size++] = 0;
        param[size++] = 0;

        //size += 2;

        byte e = isEnabled ? (byte)1 : (byte)0;
        param[size++] = e; 

        param[size++] = 0x6F; //CRC16 任意でOK
        param[size++] = 0x43; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }



}
